@@ -20,7 +20,7 @@
 #include "defs.h"
 
 #include "gdb_assert.h"
-#include "gdb_string.h"
+#include <string.h>
 
 #include "osabi.h"
 #include "arch-utils.h"
@@ -63,7 +63,6 @@ static const char * const gdb_osabi_names[] =
   "Windows CE",
   "DJGPP",
   "Irix",
-  "Interix",
   "HP/UX ELF",
   "HP/UX SOM",
   "QNX Neutrino",
