@@ -1,5 +1,5 @@
 /* Simulator for Atmel's AVR core.
-   Copyright (C) 2009, 2010, 2011 Free Software Foundation, Inc.
+   Copyright (C) 2009-2013 Free Software Foundation, Inc.
    Written by Tristan Gingold, AdaCore.
 
    This file is part of GDB, the GNU debugger.
@@ -870,7 +870,7 @@ sim_resume (SIM_DESC sd, int step, int signal)
   if (step)
     {
       cpu_exception = sim_stopped;
-      cpu_signal = TARGET_SIGNAL_TRAP;
+      cpu_signal = GDB_SIGNAL_TRAP;
     }
   else
     cpu_exception = sim_running;
@@ -1000,7 +1000,7 @@ sim_resume (SIM_DESC sd, int step, int signal)
 	case OP_break:
 	  /* Stop on this address.  */
 	  cpu_exception = sim_stopped;
-	  cpu_signal = TARGET_SIGNAL_TRAP;
+	  cpu_signal = GDB_SIGNAL_TRAP;
 	  pc = ipc;
 	  break;
 
@@ -1763,7 +1763,7 @@ int
 sim_stop (SIM_DESC sd)
 {
   cpu_exception = sim_stopped;
-  cpu_signal = TARGET_SIGNAL_INT;
+  cpu_signal = GDB_SIGNAL_INT;
   return 1;
 }
 
@@ -1855,7 +1855,7 @@ sim_set_callbacks (host_callback *ptr)
 }
 
 char **
-sim_complete_command (SIM_DESC sd, char *text, char *word)
+sim_complete_command (SIM_DESC sd, const char *text, const char *word)
 {
   return NULL;
 }
